/*
   Original Author: bbum

   Copyright 1999 by CodeFab, Inc.   All rights reserved.

   This notice may not be removed from this source code.
   The use and distribution of this software is governed by the
   terms of the MiscKit license agreement.  Refer to the license
   document included with the MiscKit distribution for the terms.
   Details of the license agreement may be found at
   www.misckit.com or within the MiscKit distribution.

   There may be more information about this software on
   www.codefab.com.  Send queries to ask_codefab@codefab.com.
*/

// this is a rather silly way to do header files
#import <WebObjects/WebObjects.h>
#import "Size.h"
#import "Size+BusinessLogic.h"
#import "Thing.h"
#import "Thing+BusinessLogic.h"

@implementation Thing (BusinessLogic)
- (NSException *)validateNameOfThing:(NSString *)aValue;
{
    NSException *returnException;

// we can only validate aValue at this point... in other words, we can't validate anything that requires knowledge about any other attribute or relationship on the object.
    if ( (aValue == nil) || ([aValue isEqualToString: @""] == YES) || ([aValue length] == 0) )
        returnException = [NSException validationExceptionWithFormat: @"Must specify a Name for the Thing."];
    else
        returnException = nil;

    [WOApp logWithFormat: @"%@ - %@ %@ [%@] -> %@%@ %@", [self entityName], NSStringFromSelector(_cmd), aValue, [aValue class], returnException ? [returnException name] : @"<valid>", returnException ? @":" : @"", returnException ? [returnException reason] : @""];

    return returnException;
}

- (NSException *) validateSizeOfThing: (Size *) aSize;
{
    NSException *returnException = nil;

    // this is more of an assertion than validation... if aSize is NOT an instance of the Size class, we raise an exception that will completely bypass the EO Validation mechanism.
    if ( (aSize != nil) && ([aSize isKindOfClass: [Size class]] == NO) )
        [NSException raise: NSInvalidArgumentException
                    format: @"Catastrophic validation failure.  The Size of a Thing must be an instance of Size.  Encountered an instance of %@.", NSStringFromClass([aSize class])];

    // a nil aSize passes this validation.  But will *not* pass the validateForSave.  This gives greater flexibility in implementing an editor using this business logic.  Other than verifying that aSize is actually an instance of the Size class, there isn't any other validation to be done at this point because we can't assume anything about the state of the rest of the object.

    [WOApp logWithFormat: @"%@ - %@ %@ [%@] -> %@%@ %@", [self entityName], NSStringFromSelector(_cmd), aSize, [aSize class], returnException ? [returnException name] : @"<valid>", returnException ? @":" : @"", returnException ? [returnException reason] : @""];

    return returnException;
}

- (NSException *) validateForSave;
{
    NSException *superException;
    NSException *localException = nil;
    NSException *returnedException;

    Size *currentSize;
    NSString *currentName;

    [WOApp logWithFormat: @"==> %@ - %@ (will invoke super's -%@)", [self entityName], NSStringFromSelector(_cmd), NSStringFromSelector(_cmd)];
    
    superException = [super validateForSave];

    currentSize = [self sizeOfThing];  /* this ensures that we trigger any business logic associated with retrieval that may not be triggered by simply referring to the iVar directly.  Make this design decision early (either way), document it, then stick to it... this way might be ever so slightly slower.  However, consider the maintenance costs if one had to change *all* direct iVar references to key/value coding (or direct access like this) at some later date due to design changes... */
    currentName = [self nameOfThing]; // same thing as -sizeOfThing....

    if (currentSize == nil)
        localException = [NSException validationExceptionWithFormat: @"Thing must have a Size before it can be saved to the database."];
    else {
        NSNumber *currentVolume = [currentSize volume];

        /* we can assume that the currentName is already valid in and of itself because the invocation against super's validateForSave would have already called the validateNameOfThing method above.  As such, we simply test currentName to make sure it is, at least, not nil-- but don't generate an exception if it is (because that would have already happened in super's implementation and the describing exception will already be in superException. */

        if ( (currentName != nil) && ([currentName length] > [currentVolume intValue]) )
            localException = [NSException validationExceptionWithFormat: @"Thing's name won't fit into the selected Size's volume.  Volume must be greater than or equal to the length of the name!"];
    }

    if (superException && localException)
        returnedException = [NSException aggregateExceptionWithExceptions: [NSArray arrayWithObjects: superException, localException, nil]];
    else if (superException)
        returnedException = superException;
    else if (localException)
        returnedException = localException;
    else
        returnedException = nil;

    [WOApp logWithFormat: @"<== %@ - %@ -> %@%@ %@", [self entityName], NSStringFromSelector(_cmd), returnedException ? [returnedException name] : @"<valid>", returnedException ? @":" : @"", returnedException ? [returnedException reason] : @""];

    return returnedException;
}
@end
